@@ -42,7 +42,7 @@ import pathlib
 import typing
 from decimal import Decimal
 from json import JSONDecoder, JSONEncoder
-from typing import Any, Callable, Dict, List, Optional, Tuple, Type, Union
+from typing import TYPE_CHECKING, Any, Callable, Dict, List, Optional, Tuple, Type, Union
 
 # 3rd party
 from typing_extensions import Protocol, runtime_checkable
@@ -50,18 +50,22 @@ from typing_extensions import Protocol, runtime_checkable
 # this package
 import domdf_python_tools
 
-if typing.TYPE_CHECKING or domdf_python_tools.__docs:  # pragma: no cover
+if TYPE_CHECKING or domdf_python_tools.__docs:  # pragma: no cover
 	# 3rd party
 	from pandas import DataFrame, Series  # type: ignore
 
 	Series.__module__ = "pandas"
 	DataFrame.__module__ = "pandas"
 
+	JSONDecoder.__module__ = "json"
+	JSONEncoder.__module__ = "json"
+
 #: .. versionadded:: 1.0.0
 FrameOrSeries = typing.TypeVar("FrameOrSeries", "Series", "DataFrame")
 
 __all__ = [
 		"PathLike",
+		"PathType",
 		"AnyNumber",
 		"check_membership",
 		"JsonLibrary",
@@ -75,8 +79,21 @@ __all__ = [
 		"SupportsIndex",
 		]
 
-#: Type hint for objects that represent filesystem paths.
 PathLike = Union[str, pathlib.Path, os.PathLike]
+"""
+Type hint for objects that represent filesystem paths.
+
+.. seealso:: :py:obj:`domdf_python_tools.typing.PathType`
+"""
+
+PathType = typing.TypeVar("PathType", str, pathlib.Path, os.PathLike)
+"""
+Type variable for objects that represent filesystem paths.
+
+.. versionadded:: 2.2.0
+
+.. seealso:: :py:obj:`domdf_python_tools.typing.PathLike`
+"""
 
 AnyNumber = Union[float, int, Decimal]
 """
@@ -93,7 +110,8 @@ def check_membership(obj: Any, type_: Union[Type, object]) -> bool:
 	Check if the type of ``obj`` is one of the types in a :py:data:`typing.Union`, :class:`typing.Sequence` etc.
 
 	:param obj: The object to check the type of
-	:param type\_: A :class:`~typing.Type` that has members, such as a List, Union or Sequence.
+	:param type\_: A :class:`~typing.Type` that has members,
+		such as a :class:`typing.List`, :py:data:`typing.Union` or :py:class:`typing.Sequence`.
 	"""
 
 	return isinstance(obj, type_.__args__)  # type: ignore
